#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-

# itools-fs is a virtual filesystem based on itools and fuse
# Copyright (C) 2007 Christian Mauduit <ufoot@ufoot.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free
# Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
# MA  02110-1301  USA


# itools -> http://www.ikaaro.org/
# fuse -> http://fuse.sourceforge.net/
# FusePython -> http://fuse.sourceforge.net/wiki/index.php/FusePython

import sys
import stat
import os
import itools.uri
import itools.vfs
import fuse

class Loggable :
    def log (self, msg):
        print msg
        sys.stdout.flush()
        
class MyStat(fuse.Stat):
    def __init__(self):
        self.st_mode = 0
        self.st_ino = 0
        self.st_dev = 0
        self.st_nlink = 0
        self.st_uid = 0
        self.st_gid = 0
        self.st_size = 0
        self.st_atime = 0
        self.st_mtime = 0
        self.st_ctime = 0

class Itoolsfs ( fuse.Fuse, Loggable ):
    def __init__ ( self, uri, *args, **kw ):
        self.log("__init__(uri=%s)" % uri)
        fuse.Fuse.__init__ ( self, *args, **kw )
        self.vfs=itools.vfs.open(uri)

    def getattr(self, path):
        self.log("getattr")
        if not self.vfs.exists(path):
            return -errno.ENOENT
        st = MyStat()
        st.st_size = self.vfs.get_size(path)
        # Not so sure while this "nlink" (number of hard link)
        # is set to 2 when the path is "/", at least, it was
        # like that in the Hello World example.
        if (path == '/'):
            st.st_nlink = 2
        else:
            st.st_nlink = 1
        if self.vfs.is_folder(path):
            st.st_mode = stat.S_IFDIR | 0555
        else:
            st.st_mode = stat.S_IFREG | 0444
        return st

    def readdir(self, path, offset):
        self.log("readdir")
        names = ['.', '..']
        names.extend (self.vfs.get_names(path))
        for r in  names:
            self.log(r)
            yield fuse.Direntry(r)

    def open(self, path, flags):
        self.log("open")
        if not self.vfs.is_file(path):
            return -errno.ENOENT
        accmode = os.O_RDONLY | os.O_WRONLY | os.O_RDWR
        if (flags & accmode) != os.O_RDONLY:
            return -errno.EACCES

    def read(self, path, size, offset):
        self.log("read")
        if not self.vfs.is_file(path):
            return -errno.ENOENT
        f_len = self.vfs.get_size(path)
        if offset < f_len:
            if offset + size > f_len:
                size = f_len - offset
            f = self.vfs.open(path)
            buf = f.read()[offset:offset+size]
            f.close()
        else:
            buf = ''
        return buf

def main():
    usage="""
Itoolsfs virtual filesystem

""" + fuse.Fuse.fusage
    print sys.argv[1]
    server = Itoolsfs(sys.argv[1])

    server.parse(errex=1)
    server.main()

if __name__ == '__main__':
    log_file=open("/tmp/itoolsfs.log","w")
    sys.stdout=log_file
    err_file=open("/tmp/itoolsfs.err","w")
    sys.stderr=err_file
    main()


